/*
 * MZViewer - A viewer for MZData format Mass Spectra
 * Copyright (c) 2005 Simon Andrews

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
package uk.ac.bbsrc.babraham.mzdata;

import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.mzdata.exceptions.MZSpectrumException;

/**
 * @author andrewss
 *
 */
public class MZSpectrum implements Comparable {
    
    private int id;
    private int msLevel;
    private boolean timeWasSet;
    private float timeValue;
    private String timeUnit;
    private float [] floatMzValues =  null;
    private float [] floatIntensityValues = null;
    private double [] doubleMzValues = null;
    private double [] doubleIntensityValues = null;
    private String type;
    private float parentMZ;
    private Vector parents = new Vector();
    private Vector children = new Vector();
    
    public void setID (int id) {
        this.id = id;
    }

    public int getID () {
        return id;
    }
    
    public void setMsLevel (int msLevel) {
        this.msLevel = msLevel;
    }
    
    public int getMsLevel () {
        return msLevel;
    }
    
    public float getTime() {
        if (timeWasSet) {
    		return timeValue;
        }
        else {
            // The time fallback is to use the order of the chromatograms
            // Since spectra are numbered from 0 we start the numbering
            // of the order at 1 (to avoid problems with 0 values when drawing)
            return id+1;
        }
    }
    
    public String getTimeUnit() {
        if (timeWasSet) {
    		return timeUnit;
        }
        else {
            // The time fallback is to use the order of the chromatograms
            return "Order";
        }
    }
    
    public int getChildIndex (MZSpectrum child) {
        return children.indexOf(child);
    }
    
    public void setTime (float time, String unit) throws MZSpectrumException {
        if (unit.equals("minutes") || unit.equals("seconds")) {
            timeValue = time;
            timeUnit = unit;
            timeWasSet = true;
        }
        else {
            throw new MZSpectrumException("Time unit '"+unit+"' was not 'minutes' or 'seconds'");
        }
    }
    public void setParentMZ (float f) {
        parentMZ = f;
    }
    
    public float getParentMZ () {
        return parentMZ;
    }
    
    public void setType (String type) throws MZSpectrumException {
    		if (type.equals("discrete") || type.equals("continuous")) {
    			this.type = type;
    		}
    		else {
    			throw new MZSpectrumException("Type '"+type+"' was not 'discrete' or 'continuous'");
    		}
    }
    
    public String getType () {
    		return type;
    }
    
    public void addFloatArray (float [] array) throws MZSpectrumException {
        // We expect two arrays to be submitted, the first containing MZ data
        // and the second containing intensity values.  These arrays must be
        // the same size
        
        // We can only store either floats or doubles, not both
        if (doubleMzValues != null || doubleIntensityValues != null) {
            throw new MZSpectrumException("Double data already entered for this spectrum.  You can't mix doubles and floats");
        }
        
        // Check we've not already got a full set of data
        if (floatMzValues != null && floatIntensityValues != null) {
            throw new MZSpectrumException("Two arrays have already been entered for this spectrum");
        }
        
        if (floatMzValues == null) {
            floatMzValues = array;
        }
        
        else {
            if (floatMzValues.length != array.length) {
                throw new MZSpectrumException ("Length of new array does not match that of the previously submitted array");
            }
            floatIntensityValues = array;
        }
    }

    
    public void addDoubleArray (double [] array) throws MZSpectrumException {
        // We expect two arrays to be submitted, the first containing MZ data
        // and the second containing intensity values.  These arrays must be
        // the same size
        
        // We can only store either floats or doubles, not both
        if (floatMzValues != null || floatIntensityValues != null) {
            throw new MZSpectrumException("Float data already entered for this spectrum.  You can't mix doubles and floats");
        }
        
        // Check we've not already got a full set of data
        if (doubleMzValues != null && doubleIntensityValues != null) {
            throw new MZSpectrumException("Two arrays have already been entered for this spectrum");
        }
        
        if (doubleMzValues == null) {
            doubleMzValues = array;
        }
        
        else {
            if (doubleMzValues.length != array.length) {
                throw new MZSpectrumException ("Length of new array does not match that of the previously submitted array");
            }
            doubleIntensityValues = array;
        }
    }

    public double getMaxMZ () {
        double max = 0;
        if (floatMzValues != null) {
            for (int i=0;i<floatMzValues.length;i++) {
                if (floatMzValues[i]>max) {
                    max = floatMzValues[i];
                }
            }
        }
        else if (doubleMzValues != null) {
            for (int i=0;i<doubleMzValues.length;i++) {
                if (doubleMzValues[i]>max) {
                    max = doubleMzValues[i];
                }
            }            
        }
        return max;
    }

    public double getMinMZ () {
        double min = 0;
        if (floatMzValues != null) {
            min = floatMzValues[0];
            for (int i=1;i<floatMzValues.length;i++) {
                if (floatMzValues[i]<min) {
                    min = floatMzValues[i];
                }
            }
        }
        else if (doubleMzValues != null) {
            min = doubleMzValues[0];
            for (int i=1;i<doubleMzValues.length;i++) {
                if (doubleMzValues[i] == 0) {
                    continue;
                }
                if (doubleMzValues[i]<min) {
                    min = doubleMzValues[i];
                }
            }            
        }
        return min;
    }

    public double getMaxIntensity () {
        double max = 0;
        if (floatIntensityValues != null) {
            for (int i=0;i<floatIntensityValues.length;i++) {
                if (floatIntensityValues[i]>max) {
                    max = floatIntensityValues[i];
                }
            }
        }
        else if (doubleIntensityValues != null) {
            for (int i=0;i<doubleIntensityValues.length;i++) {
                if (doubleIntensityValues[i]>max) {
                    max = doubleIntensityValues[i];
                }
            }            
        }
        return max;
    }

    public double getMinIntensity () {
        double min = 0;
        if (floatIntensityValues != null) {
            min = floatIntensityValues[0];
            for (int i=1;i<floatIntensityValues.length;i++) {
                if (floatIntensityValues[i]<min) {
                    min = floatIntensityValues[i];
                }
            }
        }
        else if (doubleIntensityValues != null) {
            min = doubleIntensityValues[0];
            for (int i=1;i<doubleIntensityValues.length;i++) {
                if (doubleIntensityValues[i]<min) {
                    min = doubleIntensityValues[i];
                }
            }            
        }
        return min;
    }

    public double getTICValue () {
        double count = 0;
        if (floatIntensityValues != null) {
            for (int i=0;i<floatIntensityValues.length;i++) {
                count += floatIntensityValues[i];
            }
        }
        else if (doubleIntensityValues != null) {
            for (int i=0;i<doubleIntensityValues.length;i++) {
                count += doubleIntensityValues[i];
            }
        }
        else {
            System.err.println("No intensity values associated with spectrum "+id);
        }
        return count;
    }
    
    
    public Enumeration getDataEnumeration (){
        return new SpectrumEnumeration();
    }
    
    public void addChild (MZSpectrum child) {
        children.add(child);
    }
    
    public void addParent (MZSpectrum parent) {
        parents.add(parent);
    }
    
    public MZSpectrum [] getChildren () {
        return (MZSpectrum [])(children.toArray(new MZSpectrum[0]));
    }

    public MZSpectrum [] getParents () {
        return (MZSpectrum [])(parents.toArray(new MZSpectrum[0]));
    }

    public boolean hasChildren () {
        if (children.size() == 0) {
            return(false);
        }
        else {
            return true;
        }
    }

    public boolean hasParents () {
        if (parents.size() == 0) {
            return(false);
        }
        else {
            return true;
        }
    }
    
    public int compareTo(Object o) {
        // We always sort by time
        // Since time can be well below 1 unit we multiply
        // by 1000 to ensure enough granularity at an int
        // level.
        return (int)((this.getTime()*1000)-(((MZSpectrum)o).getTime()*1000));
    }

    private class SpectrumEnumeration implements Enumeration {

        int count = 0;
        boolean useDouble = false;
        
        public SpectrumEnumeration () {
            if (doubleMzValues != null) {
                useDouble = true;
            }
        }
        
        public boolean hasMoreElements() {
            if (useDouble) {
                if (doubleMzValues.length >= count+1) {
                    return true;
                }
                else {
                    return false;
                }
            }
            else {
                if (floatMzValues.length >= count+1) {
                    return true;
                }
                else {
                    return false;
                }
            }
        }


        public Object nextElement() {
            double [] returnArray = new double [2];
            
            if (useDouble) {
                returnArray[0] = doubleMzValues[count];
                returnArray[1] = doubleIntensityValues[count];
            }
            else {
                returnArray[0] = floatMzValues[count];
                returnArray[1] = floatIntensityValues[count];
            }
            
            ++count;
            
            return returnArray;

        }
        
    }


    
}
